package com.install4j.api.context;

/**
 * The wizard context allows you to modify the GUI installer or uninstaller. It is returned by
 * {@link Context#getWizardContext()} and is only functional if the installer or uninstaller
 * is running in GUI mode.
 * @author ej-technologies GmbH
 * @see Context#getWizardContext()
 */
public interface WizardContext {

    /**
     * Change the enabled status of the "Next" button at the bottom of the current screen.
     * This setting applies only to the currently displayed screen.
     * @param enabled if the button should be enabled or not
     */
    public void setNextButtonEnabled(boolean enabled);

    /**
     * Change the enabled status of the "Back" button at the bottom of the current screen.
     * This setting applies only to the currently displayed screen.
     * @param enabled if the button should be enabled or not
     */
    public void setBackButtonEnabled(boolean enabled);

    /**
     * Change the enabled status of the "Cancel" button at the bottom of the current screen.
     * This setting applies only to the currently displayed screen.
     * @param enabled if the button should be enabled or not
     */
    public void setCancelButtonEnabled(boolean enabled);

    /**
     * Change the visibility of the "Next" button at the bottom of the current screen.
     * This setting applies only to the currently displayed screen.
     * @param enabled if the button should be enabled or not
     */
    public void setNextButtonVisible(boolean enabled);

    /**
     * Change the visibility of the "Back" button at the bottom of the current screen.
     * This setting applies only to the currently displayed screen.
     * @param enabled if the button should be enabled or not
     */
    public void setBackButtonVisible(boolean enabled);

    /**
     * Change the visibility of the "Cancel" button at the bottom of the current screen.
     * This setting applies only to the currently displayed screen.
     * @param enabled if the button should be enabled or not
     */
    public void setCancelButtonVisible(boolean enabled);

    /**
     * Move to the next screen. This has the same effect as if the user clicks on the Next button.
     */
    public void pressNextButton();

    /**
     * Move to the previous screen. This has the same effect as if the user clicks on the Back button.
     */
    public void pressPreviousButton();

    /**
     * Cancel the installer or uninstaller. This has the same effect as if the user clicks on the Cancel button.
     */
    public void pressCancelButton();

    /**
     * Transfer the focus the Next button. This could be called in the "Post-activation" script of a screen.
     */
    public void focusNextButton();

    /**
     * Set the text on the "Next" button.
     * @param text the new text
     */
    public void setNextButtonText(String text);

    /**
     * Set the text on the "Previous" button.
     * @param text the new text
     */
    public void setPreviousButtonText(String text);

    /**
     * Set the text on the "Cancel" button.
     * @param text the new text
     */
    public void setCancelButtonText(String text);

    /**
     * Install a wizard index on the left side of the window to show overall installation progress.
     * See {@link WizardIndex} for more information. The first step in the wizard index will
     * be highlighted immediately. This setting is not restricted to the current screen, it remains active for subsequent screens.
     * <p>Note that a wizard index will not be shown on banner screens or on custom screens that return
     * <tt>false</tt> in {@link com.install4j.api.screens.Screen#isShowIndex()}. Is is suggested to increase the
     * window width of your installer, if you use wizard indices.
     * @param wizardIndex the new wizard index or <tt>null</tt> if the current wizard index should be removed.
     */
    public void setWizardIndex(WizardIndex wizardIndex);

    /**
     * Change the highlighted step in the currently installed wizard index.
     * If no wizard index is set, this has no effect.
     * @param key the index key of the step that should be highlighted. If the key cannot be found
     * in the current wizard index, no step will be highlighted.
     */
    public void setWizardIndexKey(String key);

}
