package com.install4j.api.update;

import java.io.Serializable;

/**
 * Contains information about available updates.
 * <p>
 * Please see the "Auto-Update" help topic in the documentation for a detailed discussion on updaters.
 * <p>
 * When building the installers, install4j creates a file <tt>updates.xml</tt> in the media file output directory.
 * The data in that file is exposed by this class. When you place the <tt>updates.xml</tt> on a web server,
 * the "Check for update" action can download it and save an instance of this class to an installer variable.
 * You can also download an update descriptor programmatically in your application by using the
 * {@link com.install4j.api.update.UpdateChecker} class.
 *
 * @author ej-technologies GmbH
 */
public interface UpdateDescriptor extends Serializable {
    /**
     * Returns all descriptor entries for the different media files of the available version.
     * @return the descriptor entries
     */
    UpdateDescriptorEntry[] getEntries();

    /**
     * An entry that has been identified as a match for the current installation and should be used for
     * choosing an update installer.
     * <p>
     * This is only non-null if the following conditions are fulfilled:
     * <ul>
     * <li> there is indeed a newer version available
     * <li> the conditions for the maximum and minimum updatable version are satisfied by the current installation
     * <li> an entry is available whose target media set ID is equal to the media file ID of the current installation.
     * Media file IDs can be shown in the media section by invoking "Media->Show IDs" from the
     * main menu.
     * </ul>
     * @return the descriptor entry suitable for updating or <tt>null</tt> if no such entry can be found
     */
    UpdateDescriptorEntry getPossibleUpdateEntry();

    /**
     * Returns the base URL from which update installers will be downloaded.
     * <p>
     * The base URL determines the value of {@link com.install4j.api.update.UpdateDescriptorEntry#getURL()} which
     * returns the URL an update installer will be downloaded from. By default, the base URL is the empty string,
     * and the update installers will be downloaded from the same directory as the <tt>updates.xml</tt> file.
     * If <tt>updates.xml</tt> and the update installers are not placed into
     * the same directory on the web server, you have to change the <tt>baseUrl</tt> attribute in the <tt>updateDescriptor</tt>
     * element in <tt>updates.xml</tt> to the appropriate URL.
     * @return the base URL
     */
    String getBaseUrl();
}
